import { Component } from '@angular/core';
import { FormControl } from '@angular/forms';
import { appRoutes } from './app-routing.module';
import { AppService, WindowSize } from './modules/shared/services/app/app.service';
import { BrowserStorageService } from './modules/shared/services/browser-storage/browser-storage.service';
import { ThemeService } from './modules/shared/services/theme/theme.service';
import { DomSanitizer, SafeStyle } from '@angular/platform-browser';
import { AuthService } from './modules/auth/services/auth/auth.service';
import { Router } from '@angular/router';
import { UserModel } from './modules/auth/models/user.model';

enum SidebarOpenEnum {
  Default,
  Open,
  Closed,
}

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {
  isMobile = true;
  title = 'CureNet';
  sidebarOpen = SidebarOpenEnum.Default;
  sidebarMode = 'over';
  defaultSidebarOpen = false;
  lang: string;
  theme: string;
  langs: string[] = [];
  themes: string[] = [];
  themeControl = new FormControl();
  langControl = new FormControl();
  appRoutes = appRoutes;
  user: UserModel;
  isSidebarHidden = false;
  dynamicToolbarComponents = [];

  get isSidebarOpen(): boolean {
    switch (this.sidebarOpen) {
      case SidebarOpenEnum.Open:
        return true;
      case SidebarOpenEnum.Closed:
        return false;
      default:
        return this.defaultSidebarOpen;
    }
  }

  constructor(
    private appService: AppService,
    private themeService: ThemeService,
    private browserStorageService: BrowserStorageService,
    private sanitizer: DomSanitizer,
    private authService: AuthService,
    private router: Router,
  ) {
    this.configureSidebarEvents();
    this.configureThemeEvents();
    this.configureLanguageEvents();
    this.configureResizeEvents();
    this.configureUserEvents();
  }

  configureUserEvents(): void {
    this.user = this.authService.getUser();
    this.authService.userChange.subscribe(user => {
        this.user = user;
    });
  }

  configureSidebarEvents(): void {
      this.isSidebarHidden = this.appService.getSidebarHidden();
      this.appService.sidebarHiddenChange.subscribe(hidden => {
          this.isSidebarHidden = hidden;
      });
      const sidebarUserPrefference = this.browserStorageService.getItem('sidebar.open');
      if (sidebarUserPrefference !== null) {
        this.sidebarOpen = sidebarUserPrefference;
      }
      this.appService.dynamicToolbarComponentsChange.subscribe(components => {
          this.dynamicToolbarComponents = components;
      });
  }

  configureThemeEvents(): void {
    this.themes = this.themeService.getThemes();
    this.themeControl.valueChanges.subscribe(theme => {
        this.themeService.setTheme(theme);
    });
    this.onThemeChanged(this.themeService.getTheme());
    this.themeService.themeChange.subscribe(theme => {
        this.onThemeChanged(theme);
    });
  }

  configureLanguageEvents(): void {
    this.langs = this.appService.getLangs();
    this.lang = this.appService.getLang();

    this.appService.changeLang(this.lang);
    this.langControl.valueChanges.subscribe(lang => {
        this.appService.changeLang(lang);
    });

    this.onLangChanged(this.appService.getLang());
    this.appService.langChange.subscribe(lang => {
        this.onLangChanged(lang);
    });
  }

  configureResizeEvents(): void {
    this.onResize(this.appService.size);
    this.appService.sizeChange.subscribe(size => {
        this.onResize(size);
    });
  }

  onThemeChanged(theme: string): void {
    if (theme === undefined) {
        return;
    }
    this.theme = theme;
    this.themeControl.setValue(theme);
  }

  onResize(size: WindowSize): void {
    if (size === undefined) {
        return;
    }
    this.isMobile = size.isMobile;
    this.defaultSidebarOpen = !size.isMobile;
    this.sidebarMode = this.isMobile ? 'over' : 'side';
  }

  onLangChanged(lang: string): void {
    if (lang === undefined) {
      return;
    }
    this.lang = lang;
    this.langControl.setValue(lang);
  }

  toggleSidebar(): void {
    this.sidebarOpen = this.isSidebarOpen ? SidebarOpenEnum.Closed : SidebarOpenEnum.Open;
    this.browserStorageService.setItem('sidebar.open', this.sidebarOpen);
  }

  onSidebarOpenedChange(opened: boolean): void {
    this.sidebarOpen = opened ? SidebarOpenEnum.Open : SidebarOpenEnum.Closed;
  }

  url(url: string): SafeStyle {
    return this.sanitizer.bypassSecurityTrustStyle(`url('${url}')`);
  }

  logout(): void {
      this.authService.logout();
      this.router.navigateByUrl('/');
  }
}
